"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.planDeployment = void 0;
const util_1 = require("@takomo/util");
const lodash_flatten_1 = __importDefault(require("lodash.flatten"));
const lodash_uniqby_1 = __importDefault(require("lodash.uniqby"));
const confirm_1 = require("./confirm");
/**
 * @hidden
 */
const planDeployment = async (holder) => {
    const { ctx, io, timer, input: { groups, targets, configSetType }, } = holder;
    if (groups.length > 0) {
        groups.forEach((groupPath) => {
            if (!ctx.hasDeploymentGroup(groupPath)) {
                throw new util_1.TakomoError(`Deployment group '${groupPath}' not found`);
            }
        });
    }
    const deploymentGroupsToLaunch = groups.length === 0
        ? ctx.rootDeploymentGroups
        : groups.reduce((collected, path) => {
            return [...collected, ctx.getDeploymentGroup(path)];
        }, new Array());
    const sortGroups = (a, b) => {
        const order = a.priority - b.priority;
        return order !== 0 ? order : a.name.localeCompare(b.name);
    };
    const groupsToLaunch = lodash_flatten_1.default(deploymentGroupsToLaunch.map((ou) => lodash_flatten_1.default(util_1.collectFromHierarchy(ou, (o) => o.children, {
        sortSiblings: sortGroups,
        filter: (o) => o.status === "active",
    }))));
    const uniqueGroupsToLaunch = lodash_uniqby_1.default(groupsToLaunch, (o) => o.path).filter((o) => o.status === "active");
    const hasConfigSets = (target) => {
        switch (configSetType) {
            case "standard":
                return target.configSets.length > 0;
            case "bootstrap":
                return target.bootstrapConfigSets.length > 0;
            default:
                throw new Error(`Unsupported config set type: ${configSetType}`);
        }
    };
    const grs = uniqueGroupsToLaunch
        .map((ou) => {
        return {
            ...ou,
            targets: ou.targets.filter((a) => a.status === "active" &&
                hasConfigSets(a) &&
                (targets.length === 0 || targets.includes(a.name))),
        };
    })
        .filter((ou) => ou.targets.length > 0);
    const hasChanges = grs.length > 0;
    if (!hasChanges) {
        timer.stop();
        io.info("No targets to deploy");
        return {
            results: [],
            timer,
            success: true,
            status: "SKIPPED",
            message: "No targets to deploy",
        };
    }
    const plan = {
        groups: grs,
        hasChanges,
    };
    return confirm_1.confirmOperation({
        ...holder,
        plan,
    });
};
exports.planDeployment = planDeployment;
//# sourceMappingURL=plan.js.map